@@ -175,29 +175,15 @@ pmapproc_change(struct svc_req *rqstp /*__unused*/, SVCXPRT *xprt, unsigned long
 	uid_t uid;
 	char uidbuf[32];
 
-#ifdef RPCBIND_DEBUG
-	if (debugging)
-	  fprintf(stderr, "%s request for (%lu, %lu) : ",
-		  op == PMAPPROC_SET ? "PMAP_SET" : "PMAP_UNSET",
-		  reg.pm_prog, reg.pm_vers);
-#endif
-	if (!svc_getargs(xprt, (xdrproc_t) xdr_pmap, (char *)&reg)) {
-		svcerr_decode(xprt);
-		return (FALSE);
-	}
-
-	if (!check_access(xprt, op, &reg, PMAPVERS)) {
-		svcerr_weakauth(xprt);
-		return (FALSE);
-	}
-		
 	/*
 	 * Can't use getpwnam here. We might end up calling ourselves
 	 * and looping.
 	 */
-	if (__rpc_get_local_uid(xprt, &uid) < 0)
+	if (__rpc_get_local_uid(xprt, &uid) < 0) {
 		rpcbreg.r_owner = "unknown";
-	else if (uid == 0)
+		if (is_localroot(svc_getrpccaller(xprt)))
+			rpcbreg.r_owner = "superuser";
+	} else if (uid == 0)
 		rpcbreg.r_owner = "superuser";
 	else {
 		/* r_owner will be strdup-ed later */
@@ -205,17 +191,30 @@ pmapproc_change(struct svc_req *rqstp /*__unused*/, SVCXPRT *xprt, unsigned long
 		rpcbreg.r_owner = uidbuf;
 	}
 
+	if (!svc_getargs(xprt, (xdrproc_t) xdr_pmap, (char *)&reg)) {
+		svcerr_decode(xprt);
+		return (FALSE);
+	}
+#ifdef RPCBIND_DEBUG
+	if (debugging)
+	  fprintf(stderr, "%s request for (%lu, %lu) : ",
+		  op == PMAPPROC_SET ? "PMAP_SET" : "PMAP_UNSET",
+		  reg.pm_prog, reg.pm_vers);
+#endif
+
+	if (!check_access(xprt, op, &reg, PMAPVERS)) {
+		svcerr_weakauth(xprt);
+		return (FALSE);
+	}
+		
 	rpcbreg.r_prog = reg.pm_prog;
 	rpcbreg.r_vers = reg.pm_vers;
 
 	if (op == PMAPPROC_SET) {
 		char buf[32];
 
-		if (reg.pm_prot == IPPROTO_UDP) {
-			rpcbreg.r_netid = udptrans;
-		} else if (reg.pm_prot == IPPROTO_TCP) {
-			rpcbreg.r_netid = tcptrans;
-		} else {
+		rpcbreg.r_netid = pmap_ipprot2netid(reg.pm_prot);
+		if (rpcbreg.r_netid == NULL) {
 			ans = FALSE;
 			goto done_change;
 		}
@@ -289,45 +288,31 @@ pmapproc_getport(struct svc_req *rqstp /*__unused*/, SVCXPRT *xprt)
 			    svc_getrpccaller(xprt));
 		fprintf(stderr, "PMAP_GETPORT req for (%lu, %lu, %s) from %s :",
 			reg.pm_prog, reg.pm_vers,
-			reg.pm_prot == IPPROTO_UDP ? "udp" : "tcp", uaddr);
+			pmap_ipprot2netid(reg.pm_prot)?: "<invalid>",
+			uaddr);
 		free(uaddr);
 	}
 #endif
 	fnd = find_service_pmap(reg.pm_prog, reg.pm_vers, reg.pm_prot);
 	if (fnd) {
-		char serveuaddr[32], *ua;
-		char *pt1, *pt2;
+		char serveuaddr[32];
 		char *netid;
 
-		if (reg.pm_prot == IPPROTO_UDP) {
-			ua = udp_uaddr;
-			netid = udptrans;
-		} else {
-			ua = tcp_uaddr; /* To get the len */
-			netid = tcptrans;
-		}
-		if (ua == NULL) {
-			goto sendreply;
-		}
-		if ((pt1 = strrchr(ua, '.')) != NULL) {
-			*pt1 = 0;
-			if ((pt2 = strrchr(ua, '.')) != NULL) {
-				*pt2 = 0;
-				snprintf(serveuaddr, sizeof serveuaddr,
-			 		"%s.%d.%d", ua,
-					(fnd->pml_map.pm_port >> 8) & 0xff,
-			 		(fnd->pml_map.pm_port) & 0xff);
-				*pt2 = '.';
-				if (is_bound(netid, serveuaddr)) {
-					port = fnd->pml_map.pm_port;
-				} else { /* this service is dead; delete it */
-					delete_prog(reg.pm_prog);
-				}
+		netid = pmap_ipprot2netid(reg.pm_prot);
+		if (netid != NULL) {
+			snprintf(serveuaddr, sizeof serveuaddr,
+				"0.0.0.0.%ld.%ld",
+				(fnd->pml_map.pm_port >> 8) & 0xff,
+				(fnd->pml_map.pm_port) & 0xff);
+
+			if (is_bound(netid, serveuaddr)) {
+				port = fnd->pml_map.pm_port;
+			} else { /* this service is dead; delete it */
+				delete_prog(reg.pm_prog);
 			}
-			*pt1 = '.';
 		}
 	}
-sendreply:
+
 	lport = port;
 	if ((!svc_sendreply(xprt, (xdrproc_t) xdr_long, (caddr_t)&lport)) &&
 			debugging) {
@@ -341,7 +326,7 @@ sendreply:
 		fprintf(stderr, "port = %d\n", port);
 #endif
 	rpcbs_getaddr(RPCBVERS_2_STAT, reg.pm_prog, reg.pm_vers,
-		reg.pm_prot == IPPROTO_UDP ? udptrans : tcptrans,
+		pmap_ipprot2netid(reg.pm_prot) ?: "<unknown>",
 		port ? udptrans : "");
 
 	return (TRUE);
@@ -372,4 +357,24 @@ pmapproc_dump(struct svc_req *rqstp /*__unused*/, SVCXPRT *xprt)
 	return (TRUE);
 }
 
+int pmap_netid2ipprot(const char *netid)
+{
+	if (!netid)
+		return 0;
+	if (strcmp(netid, tcptrans) == 0)
+		return IPPROTO_TCP;
+	if (strcmp(netid, udptrans) == 0)
+		return IPPROTO_UDP;
+	return 0;
+}
+
+char *pmap_ipprot2netid(unsigned long proto)
+{
+	if (proto == IPPROTO_UDP)
+		return udptrans;
+	if (proto == IPPROTO_TCP)
+		return tcptrans;
+	return NULL;
+}
+
 #endif /* PORTMAP */
