@@ -68,6 +68,8 @@
 #include <pwd.h>
 #include <string.h>
 #include <errno.h>
+#include <nss.h>
+#include "config.h"
 #include "rpcbind.h"
 
 /*#define RPCBIND_DEBUG*/
@@ -79,6 +81,11 @@ int doabort = 0;	/* When debugging, do an abort on errors */
 
 rpcblist_ptr list_rbl;	/* A list of version 3/4 rpcbind services */
 
+#ifdef RPCBIND_USER
+char *rpcbinduser = RPCBIND_USER;
+#else
+char *rpcbinduser = NULL;
+#endif
 
 /* who to suid to if -s is given */
 #define RUN_AS  "daemon"
@@ -149,15 +156,18 @@ main(int argc, char *argv[])
 		fprintf(stderr, "Sorry. You are not superuser\n");
 		exit(1);
 	}
+
+	/*
+	 * Make sure we use the local service file 
+	 * for service lookkups
+	 */
+	__nss_configure_lookup("services", "files");
+
 	nc_handle = setnetconfig(); 	/* open netconfig file */
 	if (nc_handle == NULL) {
 		syslog(LOG_ERR, "could not read /etc/netconfig");
 		exit(1);
 	}
-#ifdef PORTMAP
-	udptrans = "";
-	tcptrans = "";
-#endif
 
 	nconf = getnetconfigent("local");
 	if (nconf == NULL)
@@ -177,6 +187,13 @@ main(int argc, char *argv[])
 	}
 	endnetconfig(nc_handle);
 
+#ifdef PORTMAP
+	if (!udptrans)
+		udptrans = "";
+	if (!tcptrans)
+		tcptrans = "";
+#endif
+
 	/* catch the usual termination signals for graceful exit */
 	(void) signal(SIGCHLD, reap);
 	(void) signal(SIGINT, terminate);
@@ -187,11 +204,7 @@ main(int argc, char *argv[])
 	(void) signal(SIGHUP, SIG_IGN);
 	(void) signal(SIGUSR1, SIG_IGN);
 	(void) signal(SIGUSR2, SIG_IGN);
-#ifdef WARMSTART
-	if (warmstart) {
-		read_warmstart();
-	}
-#endif
+
 	if (debugging) {
 #ifdef RPCBIND_DEBUG 
 		printf("rpcbind debugging enabled.");
@@ -206,19 +219,36 @@ main(int argc, char *argv[])
         		err(1, "fork failed");
 	}
 
-	if (runasdaemon) {
+	if (runasdaemon || rpcbinduser) {
 		struct passwd *p;
+		char *id = runasdaemon ? RUN_AS : rpcbinduser;
 
-		if((p = getpwnam(RUN_AS)) == NULL) {
-			syslog(LOG_ERR, "cannot get uid of daemon: %m");
+		/*
+		 * Make sure we use the local password file
+		 * for these lookups.
+		 */
+		__nss_configure_lookup("passwd", "files");
+
+		if((p = getpwnam(id)) == NULL) {
+			syslog(LOG_ERR, "cannot get uid of '%s': %m", id);
 			exit(1);
 		}
+                if (setgid(p->pw_gid) == -1) {
+                        syslog(LOG_ERR, "setgid to '%s' (%d) failed: %m", id, p->pw_gid);
+                        exit(1);
+                }
 		if (setuid(p->pw_uid) == -1) {
-			syslog(LOG_ERR, "setuid to daemon failed: %m");
+			syslog(LOG_ERR, "setuid to '%s' (%d) failed: %m", id, p->pw_uid);
 			exit(1);
 		}
 	}
 
+#ifdef WARMSTART
+	if (warmstart) {
+		read_warmstart();
+	}
+#endif
+
 	network_init();
 
 	my_svc_run();
@@ -519,21 +549,12 @@ init_transport(struct netconfig *nconf)
 
 #ifdef PORTMAP
 	/*
-	 * Register both the versions for tcp/ip, udp/ip and local.
+	 * Register both the versions for tcp/ip, udp/ip.
 	 */
-	if (((strcmp(nconf->nc_protofmly, NC_INET) == 0 ||
-	      strcmp(nconf->nc_protofmly, NC_INET6) == 0) &&
-		(strcmp(nconf->nc_proto, NC_TCP) == 0 ||
-		strcmp(nconf->nc_proto, NC_UDP) == 0)) ||
-		(strcmp(nconf->nc_netid, "unix") == 0) ||
-		(strcmp(nconf->nc_netid, "local") == 0)) {
+	if (si.si_af == AF_INET &&
+	    (si.si_proto == IPPROTO_TCP || si.si_proto == IPPROTO_UDP)) {
 		struct pmaplist *pml;
-		if (!svc_register(my_xprt, PMAPPROG, PMAPVERS,
-			pmap_service, 0)) {
-			syslog(LOG_ERR, "could not register on %s",
-					nconf->nc_netid);
-			goto error;
-		}
+
 		pml = malloc(sizeof (struct pmaplist));
 		if (pml == NULL) {
 			syslog(LOG_ERR, "no memory!");
@@ -542,32 +563,17 @@ init_transport(struct netconfig *nconf)
 		pml->pml_map.pm_prog = PMAPPROG;
 		pml->pml_map.pm_vers = PMAPVERS;
 		pml->pml_map.pm_port = PMAPPORT;
-		if (strcmp(nconf->nc_proto, NC_TCP) == 0) {
-			if (tcptrans[0]) {
-				syslog(LOG_ERR,
-				"cannot have more than one TCP transport");
-				goto error;
-			}
+		pml->pml_map.pm_prot = si.si_proto;
+
+		switch (si.si_proto) {
+		case IPPROTO_TCP:
 			tcptrans = strdup(nconf->nc_netid);
-			pml->pml_map.pm_prot = IPPROTO_TCP;
-
-			/* Let's snarf the universal address */
-			/* "h1.h2.h3.h4.p1.p2" */
-			tcp_uaddr = taddr2uaddr(nconf, &taddr.addr);
-		} else if (strcmp(nconf->nc_proto, NC_UDP) == 0) {
-			if (udptrans[0]) {
-				syslog(LOG_ERR,
-				"cannot have more than one UDP transport");
-				goto error;
-			}
+			break;
+		case IPPROTO_UDP:
 			udptrans = strdup(nconf->nc_netid);
-			pml->pml_map.pm_prot = IPPROTO_UDP;
-
-			/* Let's snarf the universal address */
-			/* "h1.h2.h3.h4.p1.p2" */
-			udp_uaddr = taddr2uaddr(nconf, &taddr.addr);
+			break;
 		} 
-                pml->pml_next = list_pml;
+		pml->pml_next = list_pml;
 		list_pml = pml;
 
 		/* Add version 3 information */
@@ -595,6 +601,19 @@ init_transport(struct netconfig *nconf)
 		/* Also add version 2 stuff to rpcbind list */
 		rbllist_add(PMAPPROG, PMAPVERS, nconf, &taddr.addr);
 	}
+
+	/* We need to support portmap over IPv4. It makes sense to
+	 * support it over AF_LOCAL as well, because that allows
+	 * rpcbind to identify the owner of a socket much better
+	 * than by relying on privileged ports to tell root from
+	 * non-root users. */
+	if (si.si_af == AF_INET || si.si_af == AF_LOCAL) {
+		if (!svc_register(my_xprt, PMAPPROG, PMAPVERS, pmap_service, 0)) {
+			syslog(LOG_ERR, "could not register on %s",
+					nconf->nc_netid);
+			goto error;
+		}
+	}
 #endif
 
 	/* version 3 registration */
@@ -713,7 +732,7 @@ parseargs(int argc, char *argv[])
 {
 	int c;
 	oldstyle_local = 1;
-	while ((c = getopt(argc, argv, "dwah:ilLs")) != -1) {
+	while ((c = getopt(argc, argv, "dwah:ils")) != -1) {
 		switch (c) {
 		case 'a':
 			doabort = 1;	/* when debugging, do an abort on */
